import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/models/devices_model.dart';
import 'package:syncrow_web/pages/device_managment/ceiling_sensor/bloc/ceiling_bloc.dart';
import 'package:syncrow_web/pages/device_managment/ceiling_sensor/bloc/ceiling_event.dart';
import 'package:syncrow_web/pages/device_managment/ceiling_sensor/bloc/ceiling_state.dart';
import 'package:syncrow_web/pages/device_managment/ceiling_sensor/model/ceiling_sensor_model.dart';
import 'package:syncrow_web/pages/device_managment/shared/sensors_widgets/presence_display_data.dart';
import 'package:syncrow_web/pages/device_managment/shared/sensors_widgets/presence_space_type.dart';
import 'package:syncrow_web/pages/device_managment/shared/sensors_widgets/presence_static_widget.dart';
import 'package:syncrow_web/pages/device_managment/shared/sensors_widgets/presence_status.dart';
import 'package:syncrow_web/pages/device_managment/shared/sensors_widgets/presence_update_data.dart';
import 'package:syncrow_web/pages/device_managment/shared/sensors_widgets/presense_nobody_time.dart';
import 'package:syncrow_web/pages/device_managment/shared/table/description_view.dart';
import 'package:syncrow_web/pages/device_managment/shared/table/report_table.dart';
import 'package:syncrow_web/utils/constants/assets.dart';
import 'package:syncrow_web/utils/helpers/responsice_layout_helper/responsive_layout_helper.dart';

class CeilingSensorControlsView extends StatelessWidget with HelperResponsiveLayout {
  const CeilingSensorControlsView({super.key, required this.device});

  final AllDevicesModel device;

  @override
  Widget build(BuildContext context) {
    final isExtraLarge = isExtraLargeScreenSize(context);
    final isLarge = isLargeScreenSize(context);
    final isMedium = isMediumScreenSize(context);
    return BlocProvider(
      create: (context) => CeilingSensorBloc(deviceId: device.uuid ?? '')
        ..add(CeilingInitialEvent(device.uuid ?? '')),
      child: BlocBuilder<CeilingSensorBloc, CeilingSensorState>(
        builder: (context, state) {
          if (state is CeilingLoadingInitialState || state is CeilingReportsLoadingState) {
            return const Center(child: CircularProgressIndicator());
          } else if (state is CeilingUpdateState) {
            return _buildGridView(
                context, state.ceilingSensorModel, isExtraLarge, isLarge, isMedium);
          } else if (state is CeilingReportsState) {
            return ReportsTable(
              report: state.deviceReport,
              onRowTap: (index) {},
              onClose: () {
                context.read<CeilingSensorBloc>().add(BackToCeilingGridViewEvent());
              },
            );
          } else if (state is ShowCeilingDescriptionState) {
            return DescriptionView(
              description: state.description,
              onClose: () {
                context.read<CeilingSensorBloc>().add(BackToCeilingGridViewEvent());
              },
            );
          } else if (state is CeilingReportsFailedState) {
            final model = context.read<CeilingSensorBloc>().deviceStatus;
            return _buildGridView(context, model, isExtraLarge, isLarge, isMedium);
          }
          return const Center(child: Text('Error fetching status'));
        },
      ),
    );
  }

  Widget _buildGridView(BuildContext context, CeilingSensorModel model, bool isExtraLarge,
      bool isLarge, bool isMedium) {
    return GridView(
      padding: const EdgeInsets.symmetric(horizontal: 50),
      shrinkWrap: true,
      physics: const NeverScrollableScrollPhysics(),
      gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
        crossAxisCount: isLarge || isExtraLarge
            ? 3
            : isMedium
                ? 2
                : 1,
        mainAxisExtent: 140,
        crossAxisSpacing: 12,
        mainAxisSpacing: 12,
      ),
      children: [
        PresenceState(
          value: model.presenceState,
        ),
        PresenceDisplayValue(
          value: model.sportsPara.toString(),
          postfix: '',
          description: 'Sports para',
        ),
        PresenceDisplayValue(
          value: model.presenceRange.toString(),
          postfix: 'm',
          description: 'Detection Range',
        ),
        PresenceSpaceType(
          description: 'Space Type',
          value: model.spaceType,
          action: (String value) => context.read<CeilingSensorBloc>().add(
                CeilingChangeValueEvent(
                  code: 'scene',
                  value: value,
                ),
              ),
        ),
        PresenceUpdateData(
          value: model.sensitivity.toDouble(),
          title: 'Sensitivity:',
          minValue: 1,
          maxValue: 10,
          steps: 1,
          action: (int value) {
            context.read<CeilingSensorBloc>().add(
                  CeilingChangeValueEvent(
                    code: 'sensitivity',
                    value: value,
                  ),
                );
          },
        ),
        PresenceUpdateData(
          value: model.maxDistance.toDouble(),
          title: 'Maximum Distance:',
          minValue: 0,
          maxValue: 500,
          steps: 50,
          description: 'm',
          action: (int value) => context.read<CeilingSensorBloc>().add(
                CeilingChangeValueEvent(
                  code: 'moving_max_dis',
                  value: value,
                ),
              ),
        ),
        PresenceNoBodyTime(
          value: model.noBodyTime,
          title: 'Nobody Time:',
          description: '',
          action: (String value) => context.read<CeilingSensorBloc>().add(
                CeilingChangeValueEvent(
                  code: 'nobody_time',
                  value: value,
                ),
              ),
        ),
        GestureDetector(
          onTap: () {
            context.read<CeilingSensorBloc>().add(
                GetCeilingDeviceReportsEvent(code: 'presence_state', deviceUuid: device.uuid!));
          },
          child: const PresenceStaticWidget(
            icon: Assets.illuminanceRecordIcon,
            description: 'Presence Record',
          ),
        ),
        GestureDetector(
          onTap: () {
            context
                .read<CeilingSensorBloc>()
                .add(GetCeilingDeviceReportsEvent(code: '', deviceUuid: device.uuid!));
          },
          child: const PresenceStaticWidget(
            icon: Assets.helpDescriptionIcon,
            description: 'Help Description',
          ),
        ),
      ],
    );
  }
}
