import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/models/devices_model.dart';
import 'package:syncrow_web/pages/routines/bloc/routine_bloc/routine_bloc.dart';
import 'package:syncrow_web/pages/routines/models/device_functions.dart';
import 'package:syncrow_web/pages/routines/models/wps/wps_functions.dart';
import 'package:syncrow_web/pages/routines/models/wps/wps_operational_value.dart';
import 'package:syncrow_web/pages/routines/widgets/dialog_footer.dart';
import 'package:syncrow_web/pages/routines/widgets/dialog_header.dart';
import 'package:syncrow_web/pages/routines/widgets/routine_dialogs/wall_sensor/time_wheel.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/routines/bloc/functions_bloc/functions_bloc_bloc.dart';

class WallPresenceSensor extends StatefulWidget {
  final List<DeviceFunction> functions;
  final AllDevicesModel? device;
  final List<DeviceFunctionData>? deviceSelectedFunctions;
  final String? uniqueCustomId;
  final String? dialogType;
  final bool removeComparetors;

  const WallPresenceSensor({
    super.key,
    required this.functions,
    this.device,
    this.deviceSelectedFunctions,
    this.uniqueCustomId,
    this.dialogType,
    this.removeComparetors = false,
  });

  static Future<Map<String, dynamic>?> showWPSFunctionsDialog({
    required BuildContext context,
    required List<DeviceFunction> functions,
    AllDevicesModel? device,
    List<DeviceFunctionData>? deviceSelectedFunctions,
    String? uniqueCustomId,
    String? dialogType,
    bool removeComparetors = false,
  }) async {
    return showDialog<Map<String, dynamic>?>(
      context: context,
      builder: (context) => WallPresenceSensor(
        functions: functions,
        device: device,
        deviceSelectedFunctions: deviceSelectedFunctions,
        uniqueCustomId: uniqueCustomId,
        removeComparetors: removeComparetors,
        dialogType: dialogType,
      ),
    );
  }

  @override
  State<WallPresenceSensor> createState() => _WallPresenceSensorState();
}

class _WallPresenceSensorState extends State<WallPresenceSensor> {
  late final List<WpsFunctions> _wpsFunctions;

  @override
  void initState() {
    super.initState();
    _wpsFunctions =
        widget.functions.whereType<WpsFunctions>().where((function) {
      if (widget.dialogType == 'THEN') {
        return function.type == 'THEN' || function.type == 'BOTH';
      }
      return function.type == 'IF' || function.type == 'BOTH';
    }).toList();
  }

  @override
  Widget build(BuildContext context) {
    return BlocProvider(
      create: (_) => FunctionBloc()
        ..add(InitializeFunctions(widget.deviceSelectedFunctions ?? [])),
      child: _buildDialogContent(),
    );
  }

  Widget _buildDialogContent() {
    return AlertDialog(
      contentPadding: EdgeInsets.zero,
      content: BlocBuilder<FunctionBloc, FunctionBlocState>(
        builder: (context, state) {
          final selectedFunction = state.selectedFunction;
          return Container(
            width: selectedFunction != null ? 600 : 360,
            height: 450,
            decoration: BoxDecoration(
              color: Colors.white,
              borderRadius: BorderRadius.circular(20),
            ),
            padding: const EdgeInsets.only(top: 20),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                const DialogHeader('Presence Sensor Condition'),
                Expanded(child: _buildMainContent(context, state)),
                _buildDialogFooter(context, state),
              ],
            ),
          );
        },
      ),
    );
  }

  Widget _buildMainContent(BuildContext context, FunctionBlocState state) {
    return Row(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: [
        _buildFunctionList(context),
        if (state.selectedFunction != null) _buildValueSelector(context, state),
      ],
    );
  }

  Widget _buildFunctionList(BuildContext context) {
    return SizedBox(
      width: 360,
      child: ListView.separated(
        shrinkWrap: false,
        physics: const AlwaysScrollableScrollPhysics(),
        itemCount: _wpsFunctions.length,
        separatorBuilder: (context, index) => const Padding(
          padding: EdgeInsets.symmetric(horizontal: 40.0),
          child: Divider(color: ColorsManager.dividerColor),
        ),
        itemBuilder: (context, index) {
          final function = _wpsFunctions[index];
          return ListTile(
            leading: SvgPicture.asset(
              function.icon,
              width: 24,
              height: 24,
              placeholderBuilder: (context) => const SizedBox(
                width: 24,
                height: 24,
              ),
            ),
            title: Text(
              function.operationName,
              style: context.textTheme.bodyMedium,
            ),
            trailing: const Icon(
              Icons.arrow_forward_ios,
              size: 16,
              color: ColorsManager.textGray,
            ),
            onTap: () => context.read<FunctionBloc>().add(
                  SelectFunction(
                    functionCode: function.code,
                    operationName: function.operationName,
                  ),
                ),
          );
        },
      ),
    );
  }

  Widget _buildValueSelector(BuildContext context, FunctionBlocState state) {
    final selectedFunction = state.selectedFunction!;
    final functionData = state.addedFunctions.firstWhere(
      (f) => f.functionCode == selectedFunction,
      orElse: () => DeviceFunctionData(
        entityId: '',
        functionCode: selectedFunction,
        operationName: state.selectedOperationName ?? '',
        value: null,
      ),
    );

    return Expanded(
      child: _ValueSelector(
        selectedFunction: selectedFunction,
        functionData: functionData,
        acFunctions: _wpsFunctions,
        device: widget.device,
        dialogType: widget.dialogType!,
        removeComparators: widget.removeComparetors,
      ),
    );
  }

  Widget _buildDialogFooter(BuildContext context, FunctionBlocState state) {
    return DialogFooter(
      onCancel: () => Navigator.pop(context),
      onConfirm: state.addedFunctions.isNotEmpty
          ? () {
              context.read<RoutineBloc>().add(
                    AddFunctionToRoutine(
                      state.addedFunctions,
                      widget.uniqueCustomId!,
                    ),
                  );
              Navigator.pop(
                context,
                {'deviceId': widget.functions.first.deviceId},
              );
            }
          : null,
      isConfirmEnabled: state.selectedFunction != null,
    );
  }
}

class _ValueSelector extends StatelessWidget {
  final String selectedFunction;
  final DeviceFunctionData functionData;
  final List<WpsFunctions> acFunctions;
  final AllDevicesModel? device;
  final String dialogType;
  final bool removeComparators;

  const _ValueSelector({
    required this.selectedFunction,
    required this.functionData,
    required this.acFunctions,
    required this.device,
    required this.dialogType,
    required this.removeComparators,
  });

  @override
  Widget build(BuildContext context) {
    final selectedFn =
        acFunctions.firstWhere((f) => f.code == selectedFunction);
    final values = selectedFn.getOperationalValues();

    if (_isSliderFunction(selectedFunction)) {
      return _SliderValueSelector(
        selectedFunction: selectedFunction,
        functionData: functionData,
        device: device,
        dialogType: dialogType,
      );
    }

    return _OperationalValuesList(
      values: values,
      selectedValue: functionData.value,
      device: device,
      operationName: selectedFn.operationName,
      selectCode: selectedFunction,
    );
  }

  bool _isSliderFunction(String function) =>
      ['dis_current', 'presence_time', 'illuminance_value'].contains(function);
}

class _SliderValueSelector extends StatelessWidget {
  final String selectedFunction;
  final DeviceFunctionData functionData;
  final AllDevicesModel? device;
  final String dialogType;

  const _SliderValueSelector({
    required this.selectedFunction,
    required this.functionData,
    required this.device,
    required this.dialogType,
  });

  @override
  Widget build(BuildContext context) {
    final initialValue = functionData.value ?? 250;
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        const SizedBox(height: 20),
        _ConditionToggle(
          currentCondition: functionData.condition,
          selectCode: selectedFunction,
          device: device,
          operationName: functionData.operationName,
          selectedValue: functionData.value,
        ),
        _ValueDisplay(
          value: initialValue,
          functionCode: selectedFunction,
        ),
        const SizedBox(height: 20),
        _FunctionSlider(
          initialValue: initialValue,
          functionCode: selectedFunction,
          functionData: functionData,
          device: device,
        ),
      ],
    );
  }
}

class _ConditionToggle extends StatelessWidget {
  final String? currentCondition;
  final String selectCode;
  final AllDevicesModel? device;
  final String operationName;
  final dynamic selectedValue;

  const _ConditionToggle({
    this.currentCondition,
    required this.selectCode,
    this.device,
    required this.operationName,
    this.selectedValue,
  });

  @override
  Widget build(BuildContext context) {
    const conditions = ["<", "==", ">"];
    return ToggleButtons(
      onPressed: (index) => _updateCondition(context, conditions[index]),
      borderRadius: const BorderRadius.all(Radius.circular(8)),
      selectedBorderColor: ColorsManager.primaryColorWithOpacity,
      selectedColor: Colors.white,
      fillColor: ColorsManager.primaryColorWithOpacity,
      color: ColorsManager.primaryColorWithOpacity,
      constraints: const BoxConstraints(
        minHeight: 40.0,
        minWidth: 40.0,
      ),
      isSelected:
          conditions.map((c) => c == (currentCondition ?? "==")).toList(),
      children: conditions.map((c) => Text(c)).toList(),
    );
  }

  void _updateCondition(BuildContext context, String condition) {
    context.read<FunctionBloc>().add(
          AddFunction(
            functionData: DeviceFunctionData(
              entityId: device?.uuid ?? '',
              functionCode: selectCode,
              operationName: operationName,
              condition: condition,
              value: selectedValue,
            ),
          ),
        );
  }
}

class _ValueDisplay extends StatelessWidget {
  final dynamic value;
  final String functionCode;

  const _ValueDisplay({
    required this.value,
    required this.functionCode,
  });

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 10),
      decoration: BoxDecoration(
        color: ColorsManager.primaryColorWithOpacity.withOpacity(0.1),
        borderRadius: BorderRadius.circular(10),
      ),
      child: Text(
        _getDisplayText(),
        style: context.textTheme.headlineMedium!.copyWith(
          color: ColorsManager.primaryColorWithOpacity,
        ),
      ),
    );
  }

  String _getDisplayText() {
    final intValue = (value as num?)?.toInt() ?? 0;
    switch (functionCode) {
      case 'presence_time':
        return '$intValue Min';
      case 'dis_current':
        return '$intValue CM';
      case 'illuminance_value':
        return '$intValue Lux';
      default:
        return '$intValue';
    }
  }
}

class _FunctionSlider extends StatelessWidget {
  final dynamic initialValue;
  final String functionCode;
  final DeviceFunctionData functionData;
  final AllDevicesModel? device;

  const _FunctionSlider({
    required this.initialValue,
    required this.functionCode,
    required this.functionData,
    required this.device,
  });

  @override
  Widget build(BuildContext context) {
    final (min, max) = _getSliderRange();
    return Slider(
      value: initialValue is int ? initialValue.toDouble() : min,
      min: min,
      max: max,
      divisions: (max - min).toInt(),
      onChanged: (value) => _updateValue(context, value.toInt()),
    );
  }

  (double, double) _getSliderRange() {
    switch (functionCode) {
      case 'presence_time':
        return (0, 65535);
      case 'dis_current':
        return (1, 600);
      case 'illuminance_value':
        return (0, 10000);
      default:
        return (200, 300);
    }
  }

  void _updateValue(BuildContext context, int value) {
    context.read<FunctionBloc>().add(
          AddFunction(
            functionData: DeviceFunctionData(
              entityId: device?.uuid ?? '',
              functionCode: functionCode,
              operationName: functionData.operationName,
              value: value,
              condition: functionData.condition,
            ),
          ),
        );
  }
}

class _OperationalValuesList extends StatelessWidget {
  final List<WpsOperationalValue> values;
  final dynamic selectedValue;
  final AllDevicesModel? device;
  final String operationName;
  final String selectCode;

  const _OperationalValuesList({
    required this.values,
    required this.selectedValue,
    required this.device,
    required this.operationName,
    required this.selectCode,
  });

  @override
  Widget build(BuildContext context) {
    return operationName == 'Nobody Time'
        ? _buildTimeWheel(context)
        : ListView.builder(
            padding: const EdgeInsets.all(20),
            itemCount: values.length,
            itemBuilder: (context, index) =>
                _buildValueItem(context, values[index]),
          );
  }

  Widget _buildTimeWheel(BuildContext context) {
    final currentTotalSeconds = selectedValue as int? ?? 0;
    return TimeWheelPicker(
      initialHours: currentTotalSeconds ~/ 3600,
      initialMinutes: (currentTotalSeconds % 3600) ~/ 60,
      initialSeconds: currentTotalSeconds % 60,
      onTimeChanged: (h, m, s) => _updateTotalSeconds(context, h, m, s),
    );
  }

  Widget _buildValueItem(BuildContext context, WpsOperationalValue value) {
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          _buildValueIcon(context, value),
          Expanded(child: _buildValueDescription(value)),
          _buildValueRadio(context, value),
        ],
      ),
    );
  }

  Widget _buildValueIcon(context, WpsOperationalValue value) {
    return Column(
      children: [
        if (_shouldShowTextDescription)
          Text(value.description.replaceAll("cm", '')),
        SvgPicture.asset(value.icon, width: 25, height: 25),
      ],
    );
  }

  bool get _shouldShowTextDescription =>
      operationName == 'Far Detection' ||
      operationName == 'Motionless Detection Sensitivity';

  Widget _buildValueDescription(WpsOperationalValue value) {
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 8.0),
      child: Text(value.description),
    );
  }

  Widget _buildValueRadio(context, WpsOperationalValue value) {
    return Radio<dynamic>(
      value: value.value,
      groupValue: selectedValue,
      onChanged: (_) => _selectValue(context, value.value),
    );
  }

  void _selectValue(BuildContext context, dynamic value) {
    context.read<FunctionBloc>().add(
          AddFunction(
            functionData: DeviceFunctionData(
              entityId: device?.uuid ?? '',
              functionCode: selectCode,
              operationName: operationName,
              value: value,
            ),
          ),
        );
  }

  void _updateTotalSeconds(BuildContext context, int h, int m, int s) {
    context.read<FunctionBloc>().add(
          AddFunction(
            functionData: DeviceFunctionData(
              entityId: device?.uuid ?? '',
              functionCode: selectCode,
              operationName: operationName,
              value: h * 3600 + m * 60 + s,
            ),
          ),
        );
  }
}
