import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/common/buttons/cancel_button.dart';
import 'package:syncrow_web/pages/common/buttons/default_button.dart';
import 'package:syncrow_web/pages/spaces_management/create_subspace_model/bloc/subspace_model_bloc.dart';
import 'package:syncrow_web/pages/spaces_management/create_subspace_model/bloc/subspace_model_event.dart';
import 'package:syncrow_web/pages/spaces_management/create_subspace_model/bloc/subspace_model_state.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/subspace_template_model.dart';
import 'package:syncrow_web/utils/color_manager.dart';

class CreateSubSpaceModelDialog extends StatelessWidget {
  final bool isEdit;
  final String dialogTitle;
  final List<SubspaceTemplateModel>? existingSubSpaces;
  final void Function(List<SubspaceTemplateModel> newSubspaces)? onUpdate;

  const CreateSubSpaceModelDialog(
      {Key? key,
      required this.isEdit,
      required this.dialogTitle,
      this.existingSubSpaces,
      this.onUpdate})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final screenWidth = MediaQuery.of(context).size.width;
    final textController = TextEditingController();

    return Dialog(
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.circular(20),
      ),
      child: BlocProvider(
        create: (_) {
          final bloc = SubSpaceModelBloc();
          if (existingSubSpaces != null) {
            for (var subSpace in existingSubSpaces!) {
              bloc.add(AddSubSpaceModel(subSpace));
            }
          }
          return bloc;
        },
        child: BlocBuilder<SubSpaceModelBloc, SubSpaceModelState>(
          builder: (context, state) {
            return Container(
                color: ColorsManager.whiteColors,
                child: SizedBox(
                  width: screenWidth * 0.3,
                  child: Padding(
                    padding: const EdgeInsets.all(16.0),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      mainAxisSize: MainAxisSize.min,
                      children: [
                        Text(
                          dialogTitle,
                          style: Theme.of(context)
                              .textTheme
                              .headlineLarge
                              ?.copyWith(color: ColorsManager.blackColor),
                        ),
                        const SizedBox(height: 16),
                        Container(
                          width: screenWidth * 0.35,
                          padding: const EdgeInsets.symmetric(
                              vertical: 10.0, horizontal: 16.0),
                          decoration: BoxDecoration(
                            color: ColorsManager.boxColor,
                            borderRadius: BorderRadius.circular(10),
                          ),
                          child: Wrap(
                            spacing: 8.0,
                            runSpacing: 8.0,
                            children: [
                              ...state.subSpaces.asMap().entries.map(
                                (entry) {
                                  final index = entry.key;
                                  final subSpace = entry.value;

                                  final lowerName =
                                      subSpace.subspaceName.toLowerCase();

                                  final duplicateIndices = state.subSpaces
                                      .asMap()
                                      .entries
                                      .where((e) =>
                                          e.value.subspaceName.toLowerCase() ==
                                          lowerName)
                                      .map((e) => e.key)
                                      .toList();
                                  final isDuplicate =
                                      duplicateIndices.length > 1 &&
                                          duplicateIndices.indexOf(index) != 0;

                                  return Chip(
                                    label: Text(subSpace.subspaceName,
                                        style: Theme.of(context)
                                            .textTheme
                                            .bodySmall
                                            ?.copyWith(
                                              color: ColorsManager.spaceColor,
                                            )),
                                    backgroundColor: ColorsManager.whiteColors,
                                    shape: RoundedRectangleBorder(
                                      borderRadius: BorderRadius.circular(10),
                                      side: BorderSide(
                                        color: isDuplicate
                                            ? ColorsManager.red
                                            : ColorsManager.transparentColor,
                                        width: 0,
                                      ),
                                    ),
                                    deleteIcon: Container(
                                      width: 24,
                                      height: 24,
                                      decoration: BoxDecoration(
                                        shape: BoxShape.circle,
                                        border: Border.all(
                                          color: ColorsManager.lightGrayColor,
                                          width: 1.5,
                                        ),
                                      ),
                                      child: const Icon(
                                        Icons.close,
                                        size: 16,
                                        color: ColorsManager.lightGrayColor,
                                      ),
                                    ),
                                    onDeleted: () => context
                                        .read<SubSpaceModelBloc>()
                                        .add(RemoveSubSpaceModel(subSpace)),
                                  );
                                },
                              ),
                              SizedBox(
                                width: 200,
                                child: TextField(
                                    controller: textController,
                                    decoration: InputDecoration(
                                        border: InputBorder.none,
                                        hintText: state.subSpaces.isEmpty
                                            ? 'Please enter the name'
                                            : null,
                                        hintStyle: Theme.of(context)
                                            .textTheme
                                            .bodySmall!
                                            .copyWith(
                                                color: ColorsManager
                                                    .lightGrayColor)),
                                    onSubmitted: (value) {
                                      if (value.trim().isNotEmpty) {
                                        context.read<SubSpaceModelBloc>().add(
                                            AddSubSpaceModel(
                                                SubspaceTemplateModel(
                                                    subspaceName: value.trim(),
                                                    disabled: false)));
                                        textController.clear();
                                      }
                                    },
                                    style: Theme.of(context)
                                        .textTheme
                                        .bodyMedium
                                        ?.copyWith(
                                            color: ColorsManager.blackColor)),
                              ),
                            ],
                          ),
                        ),
                        if (state.errorMessage.isNotEmpty)
                          Padding(
                            padding: const EdgeInsets.only(bottom: 16.0),
                            child: Text(state.errorMessage,
                                style: Theme.of(context)
                                    .textTheme
                                    .bodySmall
                                    ?.copyWith(
                                      color: ColorsManager.red,
                                    )),
                          ),
                        const SizedBox(height: 16),
                        Row(
                          children: [
                            Expanded(
                              child: CancelButton(
                                label: 'Cancel',
                                onPressed: () async {
                                  Navigator.of(context).pop();
                                },
                              ),
                            ),
                            const SizedBox(width: 10),
                            Expanded(
                              child: DefaultButton(
                                onPressed: (state.errorMessage.isNotEmpty)
                                    ? null
                                    : () async {
                                        final subSpaces = context
                                            .read<SubSpaceModelBloc>()
                                            .state
                                            .subSpaces;
                                        Navigator.of(context).pop();
                                        if (onUpdate != null) {
                                          onUpdate!(subSpaces);
                                        }
                                      },
                                backgroundColor: ColorsManager.secondaryColor,
                                borderRadius: 10,
                                foregroundColor: state.errorMessage.isNotEmpty
                                    ? ColorsManager.whiteColorsWithOpacity
                                    : ColorsManager.whiteColors,
                                child: const Text('OK'),
                              ),
                            ),
                          ],
                        ),
                      ],
                    ),
                  ),
                ));
          },
        ),
      ),
    );
  }
}
